import { AfterContentChecked, ChangeDetectorRef, Component, OnInit } from '@angular/core';
import { AbstractControl, FormGroup, UntypedFormGroup } from '@angular/forms';
import { lastValueFrom } from 'rxjs';
import { Structure } from '../../models/structure.model';
import { User } from '../../models/user.model';
import { ProfileService } from '../../profile/services/profile.service';
import { AuthService } from '../../services/auth.service';
import { IndicatorService } from '../../services/indicator.service';
import { NotificationService } from '../../services/notification.service';
import { OrientationService } from '../../services/orientation.service';
import { StructureService } from '../../services/structure.service';
import { CategoryEnum } from '../../shared/enum/category.enum';
import { Category } from '../../structure-list/models/category.model';
import { Filter } from '../../structure-list/models/filter.model';
import { Module } from '../../structure-list/models/module.model';
import { SearchService } from '../../structure-list/services/search.service';
import { OrientationUtils } from '../../utils/orientationUtils';
import { Utils } from '../../utils/utils';
import { AppointmentSteps } from './enums/AppointmentSteps.enum';
import { FiltersSteps } from './enums/filtersSteps.enum';
import { GenericOrientationSteps } from './enums/genericOrientationSteps.enum';
import { HotlineMediationSteps } from './enums/hotlineMediationSteps.enum';
import { NeedsType } from './enums/needs.enum';
import { OnlineDemarche } from './enums/onlineDemarche.enum';
import { OnlineDemarchesCommonSteps } from './enums/onlineDemarchesCommonSteps.enum';
import { StructuresListSteps } from './enums/structuresListSteps.enum';
import { IAppointment } from './interfaces/appointment.interface';
import { FiltersForm } from './interfaces/filtersForm.interface';
import { IOnlineMediation } from './interfaces/onlineMediation.interface';
import {
  IOrientationIndicator,
  OrientationIndicatorStructure,
  OrientationIndicatorType,
} from './interfaces/orientationIndicator.interface';
import { StructureOrientationForm } from './interfaces/structureOrientationForm.interface';
import { StructureOrientator } from './interfaces/structureOrientator.interface';
import { MediationType } from './types/mediation.type';
import { MediationStepType } from './types/mediationStep.type';

@Component({
  selector: 'app-orientation-form-view',
  templateUrl: './orientation-form-view.component.html',
  styleUrls: ['./orientation-form-view.component.scss'],
})
export class OrientationFormViewComponent implements OnInit, AfterContentChecked {
  public static readonly MAX_STEP =
    (Math.max(
      Object.keys(HotlineMediationSteps).length,
      Object.keys(AppointmentSteps).length,
      Object.keys(StructuresListSteps).length
    ) +
      Object.keys(OnlineDemarchesCommonSteps).length) /
    2;
  public orientationUtils = new OrientationUtils();
  public utils = new Utils();
  // OnlineDemarches
  public onlineDemarcheForm: UntypedFormGroup;
  public onlineDemarcheType: OnlineDemarche;
  // Equipments
  public filtersForm: FormGroup<FiltersForm>;
  public structureOrientationForm: FormGroup<StructureOrientationForm>;
  // Orientator
  public orientator: FormGroup<StructureOrientator>;
  // Enum
  public needEnum = NeedsType;
  // Global
  public needType: NeedsType;
  public indicatorNeedType: NeedsType;
  public previousNeedType: NeedsType;
  public currentType: MediationType = OnlineDemarche.common;
  public currentStep: MediationStepType = null;
  public isLastStep = false;
  public hideNavButtons = false;
  public nbSteps: number = OrientationFormViewComponent.MAX_STEP;
  public isPageValid: boolean = false;
  public filters: Filter[] = [];
  public profile: User;
  public canDeactivate: boolean = false;
  public showLoginModal: boolean = false;
  public fullScreen: boolean = false;
  public shouldResetOrientation = false;
  // Categories
  public categories: { [key: string]: Category } = {
    age: {
      id: '',
      name: '',
      theme: '',
      modules: [],
    },
    languageAndIlliteracy: {
      id: '',
      name: '',
      theme: '',
      modules: [],
    },
    genre: {
      id: '',
      name: '',
      theme: '',
      modules: [],
    },
    handicaps: {
      id: '',
      name: '',
      theme: '',
      modules: [],
    },
  };

  // Modal canExit var
  public showConfirmationModal = false;
  private resolve: Function;

  constructor(
    public orientationService: OrientationService,
    private notificationService: NotificationService,
    private authService: AuthService,
    private profilService: ProfileService,
    private profileService: ProfileService,
    private searchService: SearchService,
    private structureService: StructureService,
    private cdref: ChangeDetectorRef,
    private indicatorService: IndicatorService
  ) {
    this.setCategories();
  }
  async ngOnInit() {
    this.orientationService.rdvUser = null;
    if (history.state.rdvUser) {
      this.orientationService.rdvUser = new User(history.state.rdvUser);
    }

    this.orientationService.rdvStructure = null;
    if (history.state.rdvStructure) {
      // Create new Structure object to call Structure functions (getTypeStructureIcon,...)
      this.orientationService.rdvStructure = new Structure(history.state.rdvStructure);
    }

    if (!this.authService.isLoggedIn()) {
      this.showLoginModal = true;
    } else {
      this.profile = await this.profileService.getProfile();

      if (this.profile?.structuresLink?.length === 1) {
        this.structureService.getStructure(this.profile.structuresLink[0]).subscribe(
          (_structure) =>
            (this.orientator = this.orientationUtils.createStructureOrientatorForm({
              structureName: _structure.structureName,
              structureMail: _structure.contactMail,
              structurePhone: _structure.contactPhone,
            }))
        );
      }
    }

    if (history.state.mediationNumerique) {
      this.currentStep = 0;
      this.needType = this.needEnum.learnSkills;
      this.setCurrentNeedType(NeedsType.learnSkills);
    }
  }

  ngAfterContentChecked() {
    this.cdref.detectChanges();
  }

  public validatePage(event: boolean) {
    this.isPageValid = event;
  }

  public validateStructureRDV(event: Structure) {
    this.onlineDemarcheForm.get('structureRDV').patchValue(event._id);
    this.nextPage();
  }

  public setStepNumber(enumLength: number): void {
    this.nbSteps = enumLength / 2;
  }

  public setCurrentNeedType(needType: NeedsType): void {
    this.needType = needType;
    this.validatePage(true);
    // Init form according to needType
    switch (needType) {
      case NeedsType.onlineDemarch:
        this.onlineDemarcheForm = this.orientationUtils.createOnlineDemarchesForm();
        break;
      case NeedsType.equipmentAccess:
      case NeedsType.equipmentBuy:
        this.filtersForm = this.orientationUtils.createFiltersForm();
        break;
      case NeedsType.learnSkills:
        this.filtersForm = this.orientationUtils.createFiltersForm();
        this.onlineDemarcheForm = this.orientationUtils.createOnlineDemarchesForm();
        break;
      default:
        throw new Error(`Not implemented NeedsType in OrientationFormViewComponent ${needType}`);
    }
  }

  public checkLastStep(): void {
    const isOnlineDemarcheOrLearnSkills =
      this.needType === NeedsType.onlineDemarch || this.needType === NeedsType.learnSkills;
    const isOnlineMediationOrientationRecap =
      isOnlineDemarcheOrLearnSkills &&
      this.currentType === OnlineDemarche.onlineMediation &&
      this.currentStep === HotlineMediationSteps.orientationRecap;
    const isStructureListOrientationRecap =
      this.currentType === OnlineDemarche.structureList && this.currentStep === StructuresListSteps.orientationRecap;
    const isAppointmentRdvEnd =
      isOnlineDemarcheOrLearnSkills &&
      this.currentType === OnlineDemarche.appointment &&
      this.currentStep === AppointmentSteps.rdvEnd;
    this.isLastStep = isOnlineMediationOrientationRecap || isStructureListOrientationRecap || isAppointmentRdvEnd;
  }

  /**
   * Handle appointment, send to server
   */
  public async handleAppointment(): Promise<void> {
    const appointment: IAppointment = {
      ...this.onlineDemarcheForm.value,
    };

    // pmrAccess filter must not be send in the onlineDemarcheType array of the appointment
    appointment.onlineDemarcheType = appointment.onlineDemarcheType.filter((item) => item['name'] != 'pmrAccess');

    await lastValueFrom(this.orientationService.createAppointment(appointment));
    this.canDeactivate = true;
  }

  /**
   * Handle online mediation, send to server then print
   */
  public async handleOnlineAppointment(): Promise<void> {
    const toCreate: IOnlineMediation = {
      ...this.onlineDemarcheForm.value,
      onlineDemarcheType: this.onlineDemarcheForm.value.filters.map((module: Module) => module.displayText),
    };
    await lastValueFrom(this.orientationService.createOnlineMediation(toCreate))
      .then(() => {
        this.canDeactivate = true;
        this.notificationService.showSuccess('Votre démarche en ligne a bien été enregistrée');
      })
      .catch((error) => {
        this.notificationService.showErrorPleaseRetry('Echec de la création de votre démarche en ligne');
      })
      .finally(() => this.printForm());
  }

  public printForm(): void {
    this.sendOrientationIndicator(this.structureOrientationForm ?? this.onlineDemarcheForm);
    window.print();
    setTimeout(() => {
      this.canDeactivate = true;
      history.pushState({ rdvStructure: null, rdvUser: null }, '', '/orientation');
      window.location.reload();
    }, 100);
  }

  public setOnlineEquipmentsForm() {
    this.currentType = OnlineDemarche.structureList;
    this.currentStep = StructuresListSteps.pmrAccess;
    // first filter screen + regular StructureList form
    this.setStepNumber(Object.keys(StructuresListSteps).length);
    this.structureOrientationForm = this.orientationUtils.createStructureOrientationForm(
      this.filtersForm.value.filters,
      this.orientator
    );
  }

  public setOnlineProceduresForm() {
    switch (this.onlineDemarcheForm.get('accompanimentType').value) {
      case OnlineDemarche.structureList:
        this.currentType = OnlineDemarche.structureList;
        this.currentStep = StructuresListSteps.pmrAccess;
        this.setStepNumber(Object.keys(StructuresListSteps).length);
        this.onlineDemarcheForm = this.orientationUtils.createStructureOrientationForm(
          this.utils.convertFiltersToModule(this.filters)
        );
        break;
      case OnlineDemarche.appointment:
        this.currentType = OnlineDemarche.appointment;
        // Skip first screen appointment info for learnSkills
        if (this.indicatorNeedType == NeedsType.learnSkills) {
          this.currentStep = AppointmentSteps.infoScreen + 1;
        } else {
          this.currentStep = AppointmentSteps.infoScreen;
        }
        this.setStepNumber(Object.keys(AppointmentSteps).length);
        this.onlineDemarcheForm = this.orientationUtils.createAppointmentForm(this.filters, this.orientator);
        break;
      case OnlineDemarche.onlineMediation:
        this.currentType = OnlineDemarche.onlineMediation;
        this.currentStep = HotlineMediationSteps.infoScreen;
        this.setStepNumber(Object.keys(HotlineMediationSteps).length);
        this.onlineDemarcheForm = this.orientationUtils.createOnlineMediationForm(
          this.utils.convertFiltersToModule(this.filters)
        );
        break;
      default:
        throw new Error('Not implemented value in nextPage()');
    }
  }

  private async setCategories(): Promise<void> {
    const categories = await this.searchService.getCategories().toPromise();
    categories.forEach((categ) => {
      switch (categ.id) {
        case CategoryEnum.age: {
          this.categories.age = categ;
          break;
        }
        case CategoryEnum.languageAndIlliteracy: {
          this.categories.languageAndIlliteracy = categ;
          break;
        }
        case CategoryEnum.genre: {
          this.categories.genre = categ;
          break;
        }
        case CategoryEnum.handicaps: {
          this.categories.handicaps = categ;
          break;
        }
      }
    });
  }

  private skipStructureOrientator(isNextPage: boolean) {
    if (this.profile?.structuresLink?.length === 1) {
      // skip structureOrientator if user with only one structure
      if (isNextPage) {
        this.currentStep++;
      } else {
        this.currentStep--;
      }
    }
  }

  public setOnlineProcedureFilters(): void {
    // Handle online procedures
    const onlineProcedures: Filter[] = this.onlineDemarcheForm.value.onlineDemarcheType.map((module: Module) => {
      return new Filter('onlineProcedures', module.id, module.displayText);
    });
    if (onlineProcedures.length > 0) this.filters = [...onlineProcedures, ...this.filters];
  }

  public setFilters(category: string): void {
    // Handle online procedures
    this.filters = this.filtersForm.value.filters.map((module: Module) => {
      return new Filter(category, module.id, module.displayText);
    });
  }

  public async nextPage(): Promise<void> {
    this.isPageValid = false;
    this.hideNavButtons = false;
    // Handle filters form
    if (
      this.isFilterForm() &&
      this.currentType === GenericOrientationSteps.common &&
      this.currentStep === FiltersSteps.filterChoice
    ) {
      this.indicatorNeedType = this.needType;
      switch (this.needType) {
        case NeedsType.equipmentAccess:
          this.previousNeedType = NeedsType.equipmentAccess;
          this.setFilters('selfServiceMaterial');
          // Switch to structure list form
          this.setOnlineEquipmentsForm();
          return;
        case NeedsType.equipmentBuy:
          this.previousNeedType = NeedsType.equipmentBuy;
          this.setFilters('solidarityMaterial');
          // Switch to structure list form
          this.setOnlineEquipmentsForm();
          return;
        case NeedsType.learnSkills:
          this.previousNeedType = NeedsType.learnSkills;
          this.setFilters('baseSkills');
          this.needType = NeedsType.onlineDemarch;
          this.indicatorNeedType = NeedsType.learnSkills; // ugly but would need a big refacto to stop changing needtype to fit onlinedemarch form
          if (this.orientationService.rdvStructure || this.orientationService.rdvUser) {
            // if rdv from structure details or user, continue function below to skip accompanimentType and init onlineDemarcheForm
            this.currentStep = OnlineDemarchesCommonSteps.accompanimentType - 1;
          } else {
            this.currentStep = OnlineDemarchesCommonSteps.accompanimentType;
            return;
          }
          break;
      }
    }
    // Online demarches or learn skills
    if (this.needType === NeedsType.onlineDemarch || this.needType === NeedsType.learnSkills) {
      if (
        this.currentType === OnlineDemarche.common &&
        this.currentStep === OnlineDemarchesCommonSteps.accompanimentType - 1 &&
        (this.orientationService.rdvStructure || this.orientationService.rdvUser)
      ) {
        this.onlineDemarcheForm.get('accompanimentType').patchValue(OnlineDemarche.appointment);
        this.currentStep++;
      }

      if (
        this.currentType === OnlineDemarche.common &&
        this.currentStep === OnlineDemarchesCommonSteps.accompanimentType
      ) {
        this.setOnlineProcedureFilters();
        this.setOnlineProceduresForm();
        this.previousNeedType = this.needType;
        return;
      }
      // Handle last screen Online appointment and print
      if (
        this.currentType === OnlineDemarche.onlineMediation &&
        this.currentStep === HotlineMediationSteps.orientationRecap
      ) {
        await this.handleOnlineAppointment();
        return;
      }
      // Handle last screen appointment
      if (this.currentType === OnlineDemarche.appointment && this.currentStep === AppointmentSteps.rdvEnd - 1) {
        try {
          await this.handleAppointment();
        } catch (e) {
          console.log(e);
          this.notificationService.showErrorPleaseRetry("Echec de l'envoi de votre demande de RDV");
          // If appointment fails, don't go to next page
          return;
        }
      }
    }

    // Structure list handling
    if (this.currentType === OnlineDemarche.structureList) {
      // Print last screen
      if (this.currentStep === StructuresListSteps.orientationRecap) {
        this.printForm();
        return;
      }
      // Carto display case for full screen
      // Set to full screen
      if (this.currentStep === StructuresListSteps.address) {
        this.fullScreen = true;
      }
      // Unset fullscreen
      if (this.currentStep === StructuresListSteps.structureChoice) {
        this.fullScreen = false;
        this.skipStructureOrientator(true);
      }
    }

    // Appointment handling
    if (this.currentType === OnlineDemarche.appointment) {
      if (this.currentStep === AppointmentSteps.structureOrientator - 1) {
        this.skipStructureOrientator(true);
      }

      // If no structure found, go to the end
      if (this.currentStep === AppointmentSteps.pmrAccess) {
        if (
          (await lastValueFrom(this.structureService.getStructures(this.filters))).filter(
            (structure) => structure.hasUserWithAppointmentDN
          ).length == 0
        ) {
          this.sendOrientationIndicator(this.structureOrientationForm ?? this.onlineDemarcheForm);
          this.currentStep = AppointmentSteps.rdvEnd - 1;
        }
      }

      // Carto display case for full screen
      // Set to full screen
      if (this.currentStep === AppointmentSteps.location) {
        this.fullScreen = true;
      }
      // Unset fullscreen
      if (this.currentStep === AppointmentSteps.carto) {
        this.fullScreen = false;
      }

      // after last page, go to new orientation
      if (this.currentStep === AppointmentSteps.rdvEnd) {
        this.sendOrientationIndicator(this.structureOrientationForm ?? this.onlineDemarcheForm);
        history.pushState({ rdvStructure: null, rdvUser: null }, '', '/orientation');
        window.location.reload();
        return;
      }
    }

    if (
      this.currentType === OnlineDemarche.appointment &&
      this.currentStep === AppointmentSteps.pmrAccess - 1 &&
      (this.orientationService.rdvStructure || this.orientationService.rdvUser)
    ) {
      if (this.orientationService.rdvStructure) {
        this.onlineDemarcheForm.get('structureRDV').patchValue(this.orientationService.rdvStructure._id);
      }
      // skip pmrAccess, location and carto
      this.currentStep += 3;
    }

    // Default case
    if (this.currentStep === null) {
      // handle the form start
      this.currentStep = 0;
      return;
    }
    if (this.currentStep < this.nbSteps) {
      this.currentStep++;
      this.checkLastStep();
      this.checkHideNavButtons();
    }
  }

  public prevPage(): void {
    this.hideNavButtons = false;
    // Structure list previous page
    if (this.currentType === OnlineDemarche.structureList) {
      if (this.currentStep === StructuresListSteps.structureChoice) {
        this.fullScreen = false;
      }
      if (this.currentStep === StructuresListSteps.structureOrientator + 1) {
        this.skipStructureOrientator(false);
      }
      if (this.currentStep === StructuresListSteps.structureChoice + 1) {
        this.fullScreen = true;
      }
    }

    // Appointment previous page
    if (this.currentType === OnlineDemarche.appointment) {
      if (
        this.currentStep === AppointmentSteps.pmrAccess + 1 &&
        (this.orientationService.rdvStructure || this.orientationService.rdvUser)
      ) {
        this.currentStep--;
      }
      if (this.currentStep === AppointmentSteps.structureOrientator + 1) {
        this.skipStructureOrientator(false);
      }
      if (this.currentStep === AppointmentSteps.carto) {
        this.fullScreen = false;
      }
      if (this.currentStep === AppointmentSteps.makeAppointment) {
        this.fullScreen = true;
      }
      if (
        this.currentStep === AppointmentSteps.makeAppointment &&
        (this.orientationService.rdvStructure || this.orientationService.rdvUser)
      ) {
        this.fullScreen = false;
        this.currentStep -= 3;
        this.skipStructureOrientator(false);
      }
    }

    // Default case for first form step. If there was a previous form (like filterForm) we go back to this form
    if (this.currentStep === 0) {
      switch (this.previousNeedType) {
        case NeedsType.learnSkills:
        case NeedsType.onlineDemarch:
          this.currentType = OnlineDemarche.common;
          this.currentStep = OnlineDemarchesCommonSteps.accompanimentType;
          this.onlineDemarcheForm = this.orientationUtils.createOnlineDemarchesForm();
          this.previousNeedType = null;
          if (this.orientationService.rdvStructure || this.orientationService.rdvUser) {
            this.currentStep--;
          }
          return;
        case NeedsType.equipmentAccess:
        case NeedsType.equipmentBuy:
          this.needType = this.previousNeedType;
          this.currentType = OnlineDemarche.common;
          this.currentStep = OnlineDemarchesCommonSteps.accompanimentType;
          this.previousNeedType = null;
          break;
        default:
          // Last step, reset all
          this.reset();
          return;
      }
      // return;
      // handle the form start
    }
    if (this.currentStep > 0) {
      this.currentStep--;
      this.checkLastStep();
      this.checkHideNavButtons();
    }
  }

  public reset(): void {
    this.onlineDemarcheForm = undefined;
    this.onlineDemarcheType = undefined;
    this.filtersForm = undefined;
    this.currentStep = null;
    this.currentType = OnlineDemarche.common;
    this.isLastStep = false;
    this.shouldResetOrientation = false;
    this.nbSteps = OrientationFormViewComponent.MAX_STEP;
    this.isPageValid = false;
    this.needType = undefined;
    this.filters = [];
  }

  /**
   * Return if it's a filter form
   * @returns
   */
  private isFilterForm(): boolean {
    return (
      this.needType === NeedsType.equipmentAccess ||
      this.needType === NeedsType.equipmentBuy ||
      this.needType === NeedsType.learnSkills
    );
  }

  public canExit(): Promise<boolean> {
    // Avoid confirmation when user submit form and leave.
    if (this.canDeactivate || this.currentStep === null) {
      return new Promise((resolve) => resolve(true));
    } else {
      return new Promise((resolve) => this.showModal(resolve));
    }
  }
  private showModal(resolve: Function): void {
    this.showConfirmationModal = true;
    this.resolve = resolve;
  }
  public hasRedirectionAccepted(hasAccept: boolean): void {
    this.resolve(hasAccept);
    if (hasAccept) {
      this.sendOrientationIndicator(this.structureOrientationForm ?? this.onlineDemarcheForm);
    }
    this.showConfirmationModal = false;
  }

  private targetStructures(structureChoice: AbstractControl): OrientationIndicatorStructure[] {
    const targetStructures: OrientationIndicatorStructure[] = [];
    if (structureChoice && structureChoice.value) {
      structureChoice.value.forEach((structure: Structure) => {
        targetStructures.push({
          nom: structure.structureName,
          adresse: structure.address,
          zoneCTM: structure.categories.ctm?.join(',') ?? null,
          distance: structure.distance ?? null,
        });
      });
    }
    return targetStructures;
  }

  private displayTextNeedType(needType: NeedsType): string {
    switch (needType) {
      case NeedsType.equipmentAccess:
        return 'accès matériel';
      case NeedsType.equipmentBuy:
        return 'achat matériel';
      case NeedsType.learnSkills:
        return 'compétences';
      case NeedsType.onlineDemarch:
        return 'démarche en ligne';
      default:
        return '';
    }
  }

  private orientationType(filters: Filter[]): OrientationIndicatorType {
    const orientationType: OrientationIndicatorType = {
      typeOrientation: this.displayTextNeedType(this.indicatorNeedType ?? this.needType),
      filtres: [],
      typeAccompagnement: this.currentType,
    };
    filters.forEach((filter: Filter) => {
      orientationType.filtres.push(filter.text);
    });
    return orientationType;
  }

  private getProgress(): number {
    if (this.isLastStep) {
      return 100;
    }
    return Math.round((this.currentStep / this.nbSteps) * 100);
  }

  private async sendOrientationIndicator(orientationForm: FormGroup): Promise<void> {
    let isConnected: boolean = false;
    let profile: User = null;
    if (this.authService.isLoggedIn()) {
      isConnected = true;
      profile = await this.profilService.getProfile();
    }

    const orientationCompleted: boolean = this.isLastStep;
    const orientationIndicator: IOrientationIndicator = {
      origin: {
        nom: this.utils.nullifyEmpty(orientationForm?.get('structureOrientator')?.value.structureName),
        prescripteur: this.utils.nullifyEmpty(profile?.job?.name),
        adresse: this.utils.nullifyEmpty(orientationForm?.get('address')?.value),
      },
      target: this.utils.nullifyEmpty(this.targetStructures(orientationForm.get('structureChoice'))),
      type: this.orientationType(this.filters),
      connected: isConnected,
      completed: orientationCompleted,
      progress: this.getProgress(),
    };

    this.indicatorService.createOrientationIndicator(orientationIndicator).subscribe();
  }
  public checkHideNavButtons(): void {
    if (
      (this.needType === NeedsType.onlineDemarch || this.needType === NeedsType.learnSkills) &&
      this.currentType === OnlineDemarche.appointment &&
      this.currentStep === AppointmentSteps.carto
    ) {
      this.hideNavButtons = true;
    }
  }
  public setResetOrientation(): void {
    this.shouldResetOrientation = true;
  }
}
